// constants
const CLASS_ID = Components.ID("{68ab250e-9aba-11dc-8314-0800200c9a66}");
const CLASS_NAME = "Open IT Online Component";
const CONTRACT_ID = "@openitonline.com/manager;1";

const GOOGLEDOCS_USER_AGENT = " GoogleToolbarFF";

//class constructor
function OpenITOnlineManager() {
    this.wrappedJSObject = this;
    this.setup();
};

//class definition
OpenITOnlineManager.prototype = {

  mPrefService: Components.classes["@mozilla.org/preferences-service;1"]
                       .getService(Components.interfaces.nsIPrefBranch),
  
  QueryInterface: function(aIID) {
    if (!aIID.equals(Components.interfaces.nsIObserver) &&
        !aIID.equals(Components.interfaces.nsISupports))
      throw Components.results.NS_ERROR_NO_INTERFACE;

    return this;
  },
  
  dispose: function() {
    try {
      const osvr=Components.classes['@mozilla.org/observer-service;1'].getService(Components.interfaces.nsIObserverService);
      osvr.removeObserver(this, "xpcom-shutdown");
      osvr.removeObserver(this, "http-on-modify-request");
	  UninstallObserver.unregister();
    } catch(ex) {
      this.mlog("Error unregistering service as observer: "+ex);
    }
  },
  
  setup: function() {
  	const osvr=Components.classes['@mozilla.org/observer-service;1'].getService(Components.interfaces.nsIObserverService);
  
	osvr.addObserver(this,"xpcom-shutdown",false);
	osvr.addObserver(this,"http-on-modify-request",false);
	UninstallObserver.register();
  },

  mlog: function(log) {
      Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService).logStringMessage("Open IT Online: " + log);
  },
     
  observe: function (subject, topic, state) {
	
	if (topic == "xpcom-shutdown") {
          this.dispose();
	} else if (topic == "http-on-modify-request") {
          var httpChannel = subject.QueryInterface(Components.interfaces.nsIHttpChannel);
          var originalUserAgent = httpChannel.getRequestHeader("user-agent");
          var googleDocsHost = "";
          var currentHost = "";
          try {
              currentHost = httpChannel.URI.host;
          } catch (e) {
              return;
          }
          try {
              googleDocsHost = this.mPrefService.getCharPref("writely.uploadURL");
          } catch (e) {
              googleDocsHost = "http://docs.google.com/";
          }
          if (googleDocsHost.indexOf(currentHost) >= 0 && originalUserAgent.indexOf(GOOGLEDOCS_USER_AGENT) == -1) {
              httpChannel.setRequestHeader("user-agent", originalUserAgent+GOOGLEDOCS_USER_AGENT, false);
          }
    }
  }
};

function includeScript(uri) {
	Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
		.getService(Components.interfaces.mozIJSSubScriptLoader)
		.loadSubScript(uri);
}

var UninstallObserver = {
    _uninstall : false,
    
    observe : function(subject, topic, data) {
	  includeScript("chrome://openitonline/content/prefs.js");
	  includeScript("chrome://openitonline/content/util.js");
	  
      if (topic == "em-action-requested") {
        subject.QueryInterface(Components.interfaces.nsIUpdateItem);
    
        if (subject.id == OPENITONLINE.Util.OpenITOnline_ID) {
          if (data == "item-uninstalled") {
            this._uninstall = true;
          } else if (data == "item-cancel-action") {
            this._uninstall = false;
          }
        }
      } else if (topic == "quit-application-granted") {
        if (this._uninstall) {
          OPENITONLINE.PrefService.deleteBranch("extensions.openITOnline.");
        }
        this.unregister();
      }
    },

    register : function() {
      var observerService = Components.classes["@mozilla.org/observer-service;1"].
                             getService(Components.interfaces.nsIObserverService);
    
      observerService.addObserver(this, "em-action-requested", false);
      observerService.addObserver(this, "quit-application-granted", false);
    },

    unregister : function() {
      var observerService = Components.classes["@mozilla.org/observer-service;1"].
                             getService(Components.interfaces.nsIObserverService);
                                 
      observerService.removeObserver(this,"em-action-requested");
      observerService.removeObserver(this,"quit-application-granted");
    }
}


//class factory
var OpenITOnlineManagerFactory = {
  createInstance: function (aOuter, aIID)
  {
    if (aOuter != null)
      throw Components.results.NS_ERROR_NO_AGGREGATION;
    return (new OpenITOnlineManager()).QueryInterface(aIID);
  }
};

//module definition (xpcom registration)
var OpenITOnlineManagerModule = {
  _firstTime: true,
  registerSelf: function(aCompMgr, aFileSpec, aLocation, aType)
  {
    if (this._firstTime) {
      this._firstTime = false;
      throw Components.results.NS_ERROR_FACTORY_REGISTER_AGAIN;
    };
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.registerFactoryLocation(CLASS_ID, CLASS_NAME, CONTRACT_ID, aFileSpec, aLocation, aType);
  },

  unregisterSelf: function(aCompMgr, aLocation, aType)
  {
    aCompMgr = aCompMgr.QueryInterface(Components.interfaces.nsIComponentRegistrar);
    aCompMgr.unregisterFactoryLocation(CLASS_ID, aLocation);        
  },
  
  getClassObject: function(aCompMgr, aCID, aIID)
  {
    if (!aIID.equals(Components.interfaces.nsIFactory))
      throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

    if (aCID.equals(CLASS_ID))
      return OpenITOnlineManagerFactory;

    throw Components.results.NS_ERROR_NO_INTERFACE;
  },

  canUnload: function(aCompMgr) { return true; }
};

//module initialization
function NSGetModule(aCompMgr, aFileSpec) { return OpenITOnlineManagerModule; }